import logging
import sys

import base_sender
import threaded_sender
import user_login
import paths
import time


PRINTER_PROFILE_NAME = 'Marlin'
USB_INFO = {'VID': '2341', 'PID': '0010', 'SNR': '0001', 'COM': '/dev/null', 'EMU': True}  


class NoPrinterProfileExcepton(Exception):
    
    def __init__(self, profile_name):
        super().__init__(f'No profile with alias: {profile_name}')

def load_profile(profile_alias):
    profiles = user_login.UserLogin.load_local_printer_profiles()
    for profile in profiles:
        if profile['alias'] == profile_alias:
            return profile
    raise NoPrinterProfileExcepton
    
def test_print(print_file):
    logging.basicConfig(level=logging.DEBUG)
    fake_parent = base_sender.FakeParent()
    profile = load_profile(PRINTER_PROFILE_NAME)
    sender = threaded_sender.Sender(fake_parent, USB_INFO, profile)
    fake_parent.printer = sender
    sender.connection.resend_error_rate = 0.01
    sender.connection.error_on_line_number = 20
    sender.connection.verbose = True
    sender.connection.RECV_PAUSE_ON_ACK = 1
    sender.logger.setLevel(logging.DEBUG)
    print(f'First print. Resend range {sender.connection.RESEND_RANGE}')
    sender.gcodes(print_file)
    while sender.is_printing():
        time.sleep(0.1)
    print(f'Print finished on line {sender.current_line_number}/{sender.total_gcodes}')
    for error in sender.parent.error_history:
        print('Error registered:', error)
    time.sleep(10)
    sender.connection.RESEND_RANGE = 1
    print(f'Second print. Resend range 1')
    sender.gcodes(print_file)
    while sender.is_printing():
        time.sleep(0.1)
    for error in sender.parent.error_history:
        print('Error registered:', error)
    sender.close()
    time.sleep(1)
    print("Test finished")

def test():
    if len(sys.argv) > 1:
        filename = sys.argv[1]
    else:
        filename = "_sim.g"
    with open(filename, "rb") as f:
        test_print(f)

if __name__ == "__main__":
    test()
