import os
import sys
import time
import PIL.Image


# fix of broken paths for windows
if not sys.path:
    sys.path = []
path = os.getcwd()
if path not in sys.path:
    sys.path.insert(0, path)


import pystray
import tray_common


class PyStrayTray:

    def __init__(self):
        self.notifications_enabled = False
        self.icon = self.create_icon()
        if pystray.Icon.HAS_NOTIFICATION:
            self.status_checker = tray_common.StatusChecker(self.show_balloon)
        self.icon.run()

    def show_balloon(self, title, body):
        if pystray.Icon.HAS_NOTIFICATION:
            self.icon.notify(body, title)

    def load_icon(self):
        return PIL.Image.open(tray_common.ICON_PATH)

    def on_toggle_notifications(self, icon, item):
        self.status_checker.toggle_notifications(self.notifications_enabled)
        self.notifications_enabled = not item.checked

    def on_quit(self):
        self.status_checker.stop()
        time.sleep(self.status_checker.LOOP_SLEEP_TIME / self.status_checker.LOOP_SLEEP_TIME + 0.01)
        tray_common.stop_app()
        sys.exit()

    def on_ui(self):
        tray_common.open_browser_tab()

    def create_menus(self):
        if pystray.Icon.HAS_NOTIFICATION:
            menu = pystray.Menu(
                pystray.MenuItem('Open UI', self.on_ui, default=True),
                pystray.MenuItem('Notifications', self.on_toggle_notifications, checked=lambda _: self.notifications_enabled),
                pystray.MenuItem('Quit', self.on_quit)
            )
        else:
            menu = pystray.Menu(
                pystray.MenuItem('Open UI', self.on_ui, default=True),
                pystray.MenuItem('Quit', self.on_quit)
            )
        return menu

    def create_icon(self):
        return pystray.Icon('3DPrinterOS', icon=self.load_icon(), menu=self.create_menus())


if __name__ == '__main__':
    icon = PyStrayTray()
