from aiohttp import web
import asyncio
import importlib
import logging
import time
import os

import paths
import web_interface.page_former


PATHS_WITH_FILE_UPLOAD = {'accept_update_from_file' : paths.UPDATE_FILE_PATH,\
                          'upload_plugin' :  paths.PLUGIN_INSTALL_FILE_PATH,\
                          'upload_to_storage' : None}

EXECUTOR_TIMEOUT = 5.5


async def generic(request):
    logger = logging.getLogger('app.generic_web_handler')
    pf_args = []
    if request.method == 'GET':
        logger.debug("GET %s" % request.path)
        method_name = 'do_GET'
    elif request.method == 'POST':
        logger.debug("POST %s" % request.path)
        method_name = 'do_POST'
        post_data = await request.post()
        pf_args.append(post_data)
    else:
        raise web.HTTPForbidden()
    return await form_page_and_respond(request, method_name, pf_args)

async def reload_page_former(request):
    if request.remote == request.host.split(':')[0]:
        logging.getLogger('app.reload_page_former').info("Reloading page forment")
        importlib.reload(web_interface.page_former)
        raise web.HTTPNoContent

async def multipart_upload_handler(request):
    logger = logging.getLogger('app.multipart_upload_handler')
    error = "Unknown error"
    if request.remote == request.host.split(':')[0]:
        error = None
    else:
        if request.secure:
            cookie = request.cookies.get(web_interface.page_former.PageFormer.COOKIE_NAME_HTTPS)
        else:
            cookie = request.cookies.get(web_interface.page_former.PageFormer.COOKIE_NAME)
        if cookie in request.app['cookie_tokens']:
            error = None
        else:
            error = "Error: uploads allowed only with valid token cookie"
    file_path = PATHS_WITH_FILE_UPLOAD.get(request.path.strip("/"))
    logger.info("Upload path: %s to %s" % (request.path, file_path)) 
    if not error:
        try:
            reader = await request.multipart()
            fields_left = 128 # max number of fields
            while True:
                if request.app['3dp_app'].stop_flag:
                    raise web.HTTPFound("/")
                field = await reader.next()
                if field and field.name == 'file':
                    break
                if fields_left:
                    fields_left -= 1
                else:
                    raise RuntimeError('No upload got not file field')
        except Exception as e:
            error = "Error while searching for upload field: " + str(e)
            logger.error(error)
        else:
            if not file_path:
                filename = field.filename
                if filename:
                    file_path = os.path.join(paths.STORAGE_FOLDER, filename)
                else:
                    error = "Please select a file"
    if not error:
        size = 0
        try:
            with open(file_path, 'wb') as f:
                while True:
                    if request.app['3dp_app'].stop_flag:
                        break
                    chunk = await field.read_chunk()  # 8192 bytes by default.
                    if not chunk:
                        break
                    size += len(chunk)
                    f.write(chunk)
            if request.app['3dp_app'].stop_flag:
                os.remove(file_path)
                raise web.HTTPFound("/")
        except (OSError, IOError) as e:
            error = "Error saving upload: " + str(e)
            logger.error(error)
        else:
            logger.info('Received upload size: %dB' % size)
    pf_args = [{"multipart_error" : error}]
    return await form_page_and_respond(request, 'do_POST', pf_args)


async def form_page_and_respond(request, method_name, pf_args=[]):
    logger = logging.getLogger('app.generic_web_handler')
    pf = web_interface.page_former.PageFormer(request.app['3dp_app'],
                     request.app['3dp_content'],
                     request.app['3dp_config'],
                     request.host,
                     request.method,
                     request.raw_path,
                     request.secure,
                     request.remote,
                     request.cookies,
                     request.app['cookie_tokens'],
                     request.app['wizard_stage'],
                     request.app['wizard_enabled'])
    try:
        executor_future = asyncio.get_event_loop().run_in_executor(request.app['threads_pool'], getattr(pf, method_name), *pf_args)
        await asyncio.wait_for(executor_future, EXECUTOR_TIMEOUT)
    except asyncio.TimeoutError:
        logger.warning(f'Executor timeout when handling {request.raw_path}')
        pf.status = 504
    except Exception as e:
        logger.error(e) # check why some error are not logged by log.log_exception
    if pf.wizard_former.current_stage != request.app['wizard_stage']:
        logger.info("Wizard stage: " + str(pf.wizard_former.current_stage))
        request.app['wizard_stage'] = pf.wizard_former.current_stage
    if pf.new_cookie_token:
        request.app['cookie_tokens'].append(pf.new_cookie_token)
    request.app['wizard_enabled'] = pf.wizard_enabled
    return web.Response(status=pf.status, headers=pf.headers, body=pf.body)
